package PureMessage::HTTPD::Services::Access;
use PureMessage::Resources::Location;
use strict;
use warnings;
use base 'Exporter';
use PureMessage::Resources::DBH qw(DBH);
use Socket;
use Socket6 qw(inet_pton);
use Sophos::IP::Iface qw(is_loopback);

our @EXPORT_OK = qw(ip_is_valid
                    get_all_machine_ips
                    get_machine_hostname_from_ip
                    get_mylocation);

use constant REFRESH_INTERVAL => 120;

my $last_cached_on;

my %known_ips = ();

my $hostname_by_ip = { };

my $location;

sub _is_valid_loopback_ip {
    my $ip = shift;
    my $res = inet_pton(AF_INET, $ip);

    return 0 if !defined $res;
    return is_loopback($res);
}

sub ip_is_valid {
    my $ip = shift;
    return $known_ips{$ip} 
        if defined $known_ips{$ip} && _cache_is_valid();

    if (_is_valid_loopback_ip($ip)) {
        $last_cached_on = time;
        return $known_ips{$ip} = 1;
    }

    my $dbh = DBH();
    my $sql = 'select ip from cfg_location where ip = ?';
    my $sth = $dbh->prepare($sql);
    $sth->execute($ip);
    my $ar = $sth->fetchall_arrayref;
    my $ok = @$ar > 0 ? 1 : 0;
    $last_cached_on = time; 
    return $known_ips{$ip} = $ok;
}

sub get_all_machine_ips {
    my $dbh = DBH();
    my $loc = PureMessage::Resources::Location->new();
    my $location_id = $loc->location_id;
    my $sql = "select ip from cfg_location";

    my $ar = $dbh->selectcol_arrayref($sql);
    return $ar;
}

sub get_machine_hostname_from_ip {
    my $ip = shift;
    return $hostname_by_ip->{$ip}->{hostname} if defined $hostname_by_ip->{$ip};
    my $dbh = DBH();
    my $sql = 'select ip,hostname from cfg_location';
    my $hostname_by_ip = $dbh->selectall_hashref($sql, 'ip');
    return $hostname_by_ip->{$ip}->{hostname} if defined $hostname_by_ip->{$ip};
}

sub get_mylocation {
    return $location if $location;
    eval {
        require PureMessage::Resources::Location;
        my %cfg = %PureMessage::Resources::Location::location_config;
        $location = $cfg{location} if exists $cfg{location};
    };

    warn "Unable to determine my location: $@" if $@;
    return $location;
}

sub _cache_is_valid {
    return 0 if !defined $last_cached_on;
    return ((time - $last_cached_on) < REFRESH_INTERVAL) ? 1 : 0;
}

1;
